/*
Copyright (C) 2000 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ckDialog.h"
#include "ckDialogControl.h"
#include "ckButtonControl.h"
#include "ckControlTable.h"
#include "ckControlImp.h"
#include "ckScanner.h"
#include "ckMiscParser.h"
#include "ckStream.h"
#include "ckVariables.h"
#include "ckValueWrapper.h"
#include "ckPainter.h"

//
// static 
//
ckControl::ControlStyleT ckButtonControl::_buttonstyle[] =
{
    {"bsauto3state"             , BS_AUTO3STATE},
    {"bsautocheckbox"	        , BS_AUTOCHECKBOX},
    {"bsautoradiobutton"        , BS_AUTORADIOBUTTON},
    {"bsdefpushbutton"          , BS_DEFPUSHBUTTON},
    {"bspushbutton"             , BS_PUSHBUTTON},
    {"bspushlike"               , BS_PUSHLIKE},
    {"bsgroupbox"               , BS_GROUPBOX},

    {"bsbottom"                 , BS_BOTTOM},
    {"bstop"                    , BS_TOP},

    {"bscenter"                 , BS_CENTER},
    {"bsright"                  , BS_RIGHT},
    {"bsvcenter"                , BS_VCENTER},
    
    {"bslefttext"               , BS_LEFTTEXT},
    {"bsmultiline"			    , BS_MULTILINE},
    
    {"bsnotify"                 , BS_NOTIFY},
    {"bsownerdraw"              , BS_OWNERDRAW},
    //^- this has to be set if want to intercept double click on buttons

    // not used
    {"bs3state"                 , BS_3STATE},
    {"bscheckbox"               , BS_CHECKBOX},
    {0, 0}
};



ckButtonControl::ckButtonControl()
:   ckControl(string(BUTTON_TYPE))
{

}


void ckButtonControl::createwindow(DWORD xstyle, const char *classname, const char *windowname, DWORD style,
                    int x, int y, int w, int h, HWND parent, HMENU menu, HINSTANCE hinstance)
{
    ckControl::createwindow(xstyle, classname, windowname, style,
                x, y, w, h, parent, menu, hinstance);
    if(getHWND())
    {
        if(_attributeExists(BEVELLIGHT_ATTRIB))
        {
            _bevellight.reset(new Pen(strtocolor(getAttribute(BEVELLIGHT_ATTRIB).c_str())));
            if((HGDIOBJ)(*_bevellight.get()))
            {
                if(_attributeExists(BEVELDARK_ATTRIB))
                {
                    _beveldark.reset(new Pen(strtocolor(getAttribute(BEVELDARK_ATTRIB).c_str())));
                    if(!(HGDIOBJ)(*_beveldark.get()))
                        ckLogger::instance()->log(FAIL, "Error occured while loading bevel dark color");
                }
            }       
            else
                ckLogger::instance()->log(FAIL, "Error occured while loading bevel light color");
        }
    }
}

ckError ckButtonControl::performAction(string const &action, ckValueWrapper &wrapper) 
{
    if(getHWND() != NULL)
    {
        _LSDEBUG2("ckButtonControl::performAction() action: %s", action.c_str());
        return ckControlImpMap<ckButtonImp>::instance()->execute(action, wrapper, (*this));
    }
    else
    {
        _LSDEBUG1("ckButtonControl::performAction() called when window hasn't been created");
        return FAIL;
    }
}

ckError ckButtonControl::retreiveVariable(string const &var, string &value)
{ 
    if(stricmp(var.c_str(), TITLE_ATTRIB) == 0)
    {
        //get the text of edit
        value = ckControlImpMap<ckButtonImp>::instance()->getImp()->_getText(*this);
    }
    else
    {
        return ckControl::retreiveVariable(var, value);
    }
    
    return SUCCESS;
}

/*
DWORD ckButtonControl::_parseStyle()
{
    DWORD style = WS_VISIBLE;

    _LSDEBUGFUNS("ckButtonControl::_parseStyle()");

    {
    _LSDEBUG1("ckButtonControl::_parseStyle() parsing button style");
    ckStyleParser styleparser(style, ckButtonControl::getstylegallery());
    styleparser.parse(ckStrStream(getAttribute(STYLE_ATTRIB)));
    }
    {
    _LSDEBUG1("ckButtonControl::_parseStyle() parsing dialog style");
    //cool way for data inheritance?
    ckStyleParser styleparser(style, ckDialogControl::getstylegallery());
    styleparser.parse(ckStrStream(getAttribute(STYLE_ATTRIB)));
    }

    _LSDEBUG2("ckButtonControl::_parseStyle() got button style: %08X", style);
    _LSDEBUG1("WARNING: should probably do a style check if want ot be safe");
    _LSDEBUG1("ckButtonControl::_parseStyle() we should warn user if invalid style!");
    
    if(parentHWND())
    {
        _LSDEBUG1("ckButtonControl::_parseStyle() auto set child");
        style |= WS_CHILD;
    }

    _LSDEBUG2("ckButtonControl::_parseStyle() got total style: %08X", style);
    _LSDEBUGFUNE("ckButtonControl::_parseStyle()");

    return style;
}
*/



void ckButtonControl::acceptPainter(ckPainter &painter) 
{
    painter.paintButton(*this);
}