/*
Copyright (C) 2001 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ckControlGDI.h"

///////////////////////////////////////////////
//	class GDIObject
///////////////////////////////////////////////
GDIObject::GDIObject(HDC hDC, HGDIOBJ hGdiObj)
:	_hDC(hDC),
	_hOldObj(::SelectObject(hDC, hGdiObj))
{
}

GDIObject::~GDIObject()
{
	::SelectObject(_hDC, _hOldObj);
}


///////////////////////////////////////////////
//	class StockObject
///////////////////////////////////////////////
StockObject::StockObject(HDC hDC, int GdiObj)
:	GDIObject(hDC, ::GetStockObject(GdiObj))
{}


///////////////////////////////////////////////
//	class BlackBrush
///////////////////////////////////////////////
BlackBrush::BlackBrush(HDC hDC)
:	StockObject(hDC, BLACK_BRUSH)
{}


///////////////////////////////////////////////
//	class BlackPen
///////////////////////////////////////////////
BlackPen::BlackPen(HDC hDC)
:	StockObject(hDC, BLACK_PEN)
{}

///////////////////////////////////////////////
//	class UserObject
///////////////////////////////////////////////
UserObject::UserObject(HGDIOBJ hGdiObj)
:	_hGdiObj(hGdiObj)
{}

UserObject::~UserObject()
{
	if( _hGdiObj != reinterpret_cast<HGDIOBJ>(NULL) )
		::DeleteObject(_hGdiObj);
}

///////////////////////////////////////////////
//	class Pen
///////////////////////////////////////////////
Pen::Pen(COLORREF rColor, int width, int style)
: UserObject(::CreatePen(style, width, rColor))
{}


///////////////////////////////////////////////
//	class Font
///////////////////////////////////////////////
Font::Font(const char *fontface)
: UserObject(::CreateFont(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, fontface))
{}


///////////////////////////////////////////////
//	class Brush
///////////////////////////////////////////////
Brush::Brush(COLORREF rColor)
    : UserObject(::CreateSolidBrush(rColor))
{}

///////////////////////////////////////////////
//	class UserStock
///////////////////////////////////////////////
UserStock::UserStock(HDC hDC, UserObject &rUObj)
:	GDIObject(hDC, (HGDIOBJ)rUObj)
{}


///////////////////////////////////////////////
//	class Bitmap
///////////////////////////////////////////////
Bitmap::Bitmap()
:	UserObject(reinterpret_cast<HGDIOBJ>(NULL))
{}
        
Bitmap::Bitmap(HBITMAP hBitmap)
:	UserObject(reinterpret_cast<HGDIOBJ>(hBitmap))
{	ResetDim();		}

Bitmap::Bitmap(Bitmap &rBitmap)
:	UserObject(rBitmap.Release())
{
    _Width = rBitmap._Width;
	_Height = rBitmap._Height;
}
        
void Bitmap::operator = (Bitmap &rbitmap)
{
    Free();
	_hGdiObj = rbitmap.Release();
	ResetDim();
}

void Bitmap::Compatible(HDC hDC, int width, int height)
{
	Free();
	_hGdiObj = reinterpret_cast<HGDIOBJ>
	(::CreateCompatibleBitmap(hDC, width, height));
	ResetDim();
}

BOOL Bitmap::LoadFile(const char *pFilename)
{
	Free();
	_hGdiObj = 
		::LoadImage(NULL, pFilename, IMAGE_BITMAP,
					0, 0, LR_DEFAULTCOLOR|LR_LOADFROMFILE);
	if(_hGdiObj == reinterpret_cast<HGDIOBJ>(NULL) )
        return FALSE;

	ResetDim();
    return TRUE;
}

BOOL Bitmap::Load(HINSTANCE hInstance, int ResId)
{
	Free();
	_hGdiObj = 
		::LoadImage(hInstance, MAKEINTRESOURCE(ResId), IMAGE_BITMAP,
					0, 0, LR_DEFAULTCOLOR);
	if(_hGdiObj == reinterpret_cast<HGDIOBJ>(NULL) )
		return FALSE;
	ResetDim();
    return TRUE;
}

void Bitmap::SetHBitmap(HBITMAP hBitmap)
{
	Free();
	_hGdiObj = reinterpret_cast<HGDIOBJ>(hBitmap);
	ResetDim();
}

HGDIOBJ Bitmap::Release()
{	
	HGDIOBJ temp = _hGdiObj;
	_hGdiObj = reinterpret_cast<HGDIOBJ>(NULL);
	return temp;
}
	
void Bitmap::Free()
{
	if( _hGdiObj != reinterpret_cast<HGDIOBJ>(NULL) )
	{
		::DeleteObject(_hGdiObj);
		_hGdiObj = reinterpret_cast<HGDIOBJ>(NULL);
	}
}

void Bitmap::ResetDim()
{
	if(_hGdiObj != reinterpret_cast<HGDIOBJ>(NULL))
	{
		BITMAP bm;
		::GetObject (_hGdiObj, sizeof(bm), &bm);
		_Width = bm.bmWidth;
		_Height = bm.bmHeight;
	}
}


void CopyHDC(HDC src, HDC trg, RECT &r)
{
    ::BitBlt(trg, r.left , r.top, r.right - r.left, r.bottom - r.top, src, 0, 0, SRCCOPY);
}