/*
Copyright (C) 2001 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _CK_CONTROLGDI_H_
#define _CK_CONTROLGDI_H_

#include "ckDialog.h"


///////////////////////////////////////////////
//	Color constants
//PALETTERGB(BYTE bRed,BYTE bGreen,BYTE bBlue);
///////////////////////////////////////////////
static const COLORREF BLACK_COLOR = PALETTERGB(0, 0, 0);
static const COLORREF WHITE_COLOR = PALETTERGB(0xFF, 0xFF, 0xFF);

///////////////////////////////////////////////
//	class GDIObject
///////////////////////////////////////////////
//	Basic class that takes care of the selecting
//	new object and returning back the old ones.
//  The value of the HGDIOBJ cannot be NULL
class GDIObject
{
public:
	GDIObject(HDC hDC, HGDIOBJ hGdiObj);
	virtual ~GDIObject();
protected:
	HGDIOBJ _hOldObj;
	HDC _hDC;
};

///////////////////////////////////////////////
//	class StockObject
///////////////////////////////////////////////
//	For selecting the stocked objects from the
//	system
class StockObject : public GDIObject
{
protected:
	StockObject(HDC hDC, int GdiObj);
};

///////////////////////////////////////////////
//	class BlackBrush
///////////////////////////////////////////////
class BlackBrush : public StockObject
{
public:
	BlackBrush(HDC hDC);
};

///////////////////////////////////////////////
//	class BlackPen
///////////////////////////////////////////////
class BlackPen : public StockObject
{
public:
	BlackPen(HDC hDC);
};

///////////////////////////////////////////////
//	class UserObject
///////////////////////////////////////////////
// UserObjects are user's predefined objects
// that are used later for painting or drawing.
// The value of HGDIOBJ could be NULL
class UserObject
{
public:
	UserObject(HGDIOBJ hGdiObj);
	virtual ~UserObject();

	operator HGDIOBJ () {return _hGdiObj;}
protected:
	HGDIOBJ _hGdiObj;
};

///////////////////////////////////////////////
//	class Pen
///////////////////////////////////////////////
class Pen : public UserObject
{
public:
	Pen(COLORREF rColor=BLACK_COLOR, int width=0, int style=PS_SOLID);
};

///////////////////////////////////////////////
//	class Font
///////////////////////////////////////////////
class Font : public UserObject
{
public:
    Font(const char *fontface);
};

///////////////////////////////////////////////
//	class Brush
///////////////////////////////////////////////
class Brush : public UserObject
{
public:
    Brush(COLORREF rColor);
};

///////////////////////////////////////////////
//	class UserStock
///////////////////////////////////////////////
//	UserStock is the a stock class that temporarily
//	holds the user defined objects and the old object
//	is later selected back by the GDIObject base class
class UserStock : public GDIObject
{
public:
	UserStock(HDC hDC, UserObject &rUObj);
};	


///////////////////////////////////////////////
//	class Bitmap
///////////////////////////////////////////////
class Bitmap : public UserObject
{
public:
	Bitmap();
    
	Bitmap(HBITMAP hBitmap);

	Bitmap(Bitmap &rBitmap);
    
	void operator = (Bitmap &rbitmap);

    void Compatible(HDC hDC, int width, int height);

    BOOL LoadFile(const char *pFilename);

    BOOL Load(HINSTANCE hInstance, int ResId);

	int Width() const {return _Width;}
	int Height() const {return _Height;}

	//This is bad bad! use it if you have to add a 
	//bitmap handle to this responsible class
	void SetHBitmap(HBITMAP hBitmap);

	operator HBITMAP() const {return reinterpret_cast<HBITMAP>(_hGdiObj);}

	HGDIOBJ Release();

	void Free();

protected:
	void ResetDim();
	int _Width, _Height;

};


void CopyHDC(HDC src, HDC trg, int width, int height);

#endif