/*
Copyright (C) 2001 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _CK_CONTROLSTYLE_H_
#define _CK_CONTROLSTYLE_H_

#include "ckDialog.h"
#include "ckControl.h"

// control must inherit from this to have style!
template <class T>
class ckHasStyle
{
public:
    static ckControl::ControlStyleT *getstylegallery();
};

// class must implement static ckControl::ControlStyleT *getstylegallery()
// if want to be ablet to use ckControlStyle
//
template <class T>
class ckControlStyle
{
public:
    static ckControlStyle<T> *instance()
    {
        if(_instance.get() == NULL)
        {
            _instance.reset(new ckControlStyle<T>);
        }

        assert(_instance.get());
        return _instance.get();
    }

    DWORD findstyle(string const &style)
    {
        if(_style.find(style) != _style.end())
        {
            return _style[style];
        }

        return 0;
    }

    
protected:
    static map<string, DWORD> _style;
    
    ckControlStyle()
    {
        init();
    }

    static init()
    {
        int i = 0;
        ckControl::ControlStyleT *stylegallery = T::getstylegallery();
        while(stylegallery[i]._name)
        {
            _style[stylegallery[i]._name] = stylegallery[i]._style;
            ++i;
        }

        _LSDEBUG1("Dumping style gallery");
        map<string, DWORD>::iterator it;
        for(it = _style.begin(); it != _style.end(); ++it)
        {
            _LSDEBUG2("style name: %s", it->first.c_str());
            _LSDEBUG2("style value: %08X", it->second);
        }
    }

    static auto_ptr<ckControlStyle<T> > _instance;
};
// 
// _styleParser
//
// pre  : stylestr = style attribute, dwstyle = predefined style
// post : if success dwstyle |= style attribute's style code
template<class T>
ckError _styleParser(string const &stylestr, DWORD &dwstyle)
{
    DWORD style = 0;
    string str;
    ckError error = SUCCESS;
    //dwstyle = 0;
    //ckControlStyle<T> *styler = ckControlStyle<T>::instance();

    if(!stylestr.empty())
    {
        ckStrStream stream(stylestr);
        ckStyleScanner scanner(stream);
        ckValueWrapper wrapper(scanner);
        error = wrapper.getStringWithLength(str, false);

        while(error == SUCCESS)
        {
            //style = styler->findstyle(str);
            int i = 0;
            ckControl::ControlStyleT *stylegallery = T::getstylegallery();
            while(stylegallery[i]._name && str != stylegallery[i]._name)
            {                
                ++i;
            }

            if(stylegallery[i]._name)
            {
                style = stylegallery[i]._style;
                dwstyle |= style;
            }
            else
            {
                _LSDEBUG2("_styleParser() can't find style %s", str.c_str());
            }


            _LSDEBUG1("Dumping style gallery");
            i = 0;
            while(stylegallery[i]._name)
            {
                _LSDEBUG2("style name: %s", stylegallery[i]._name);
                _LSDEBUG2("style code: %08X", (DWORD)stylegallery[i]._style);
                ++i;
            }

            error = wrapper.getStringWithLength(str, false);
        }
    }

    return error;
}


#endif 

