/*
Copyright (C) 2001 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "ckDialog.h"
#include "ckStream.h"
#include "ckScanner.h"
#include "ckValueWrapper.h"
#include "ckVariables.h"
#include "ckMiscParser.h"

//
// ckSizeParser
//
ckSizeParser::ckSizeParser(RECT &r, ckControl &control) 
:   _r(r),
    _control(control)
{}

ckError ckSizeParser::parse(ckStream &ckstream)
{
    ckIntParser intParser(_control);
    ckError error = intParser.parse(ckstream);

    if(error == SUCCESS)
    {
        _r.left = intParser.getInt();
        error = intParser.parse(ckstream);
        if(error == SUCCESS)
        {
            _r.top = intParser.getInt();
            error = intParser.parse(ckstream);
            if(error == SUCCESS)
            {
                _r.right = intParser.getInt();
                error = intParser.parse(ckstream);
                if(error == SUCCESS)
                {
                    _r.bottom = intParser.getInt();
                }
            }
        }
    }

    if(error != SUCCESS)
    {
        _LSDEBUG1("ckSizeparser()::parse() failed to parse size");
    }
    return error;
}

//
// ckIntParser
//
ckIntParser::ckIntParser(ckControl &control) 
:   _control(control)
{}

ckError ckIntParser::parse(ckStream &ckstream)
{
    string str;
    ckDialogTemplateScanner sizescanner(ckstream);
    ckValueWrapper wrapper(sizescanner);
    ckError error = wrapper.getVariableInt(_i, str);
    //assert(error == SUCCESS);
    if(_i == -1) //it's variable let's expand
    {
        error = ckVariables::instance()->translateVariable(str, &_control);
        if(error == SUCCESS)
            _i = atoi(str.c_str());
        else
        {
            _LSDEBUG1("ckIntParser::parse() invalid int / variable");
            return FAIL;
        }
    }

    return error;
}


//
// ckStyleParser
//
ckStyleParser::ckStyleParser(DWORD &style, ckControl::ControlStyleT *stylegallery)
:   _style(style),
    _stylegallery(stylegallery)
{
    assert(_stylegallery != NULL);
}

ckError ckStyleParser::parse(ckStream &ckstream)
{
    string str;
    ckStyleScanner scanner(ckstream);
    ckValueWrapper wrapper(scanner);
    ckError error = wrapper.getStringWithLength(str, false);

    while(error == SUCCESS)
    {
        int i = 0;

        // this is our special built in style
        // should do something about this ugly crap
        _LSDEBUG0;
        if(stricmp(str.c_str(), HIDDENSYLE_SYMBOL) != 0)
        {
            _LSDEBUG0;
            while(_stylegallery[i]._name && stricmp(str.c_str(), _stylegallery[i]._name) != 0)
                ++i;

            _LSDEBUG0;
            if(_stylegallery[i]._name)
            {
                _style |= _stylegallery[i]._style;
            }
            else
            {
                _LSDEBUG2("ckStyleParser::parse() can't find style: ", str.c_str());
            }
            _LSDEBUG0;
        }
        else
        {   //negate our default visible
            _LSDEBUG0;
            _style &= ~WS_VISIBLE;
        }

        _LSDEBUG0;
        error = wrapper.getStringWithLength(str, false);
    }


    _LSDEBUG2("ckStyleParser::parse() got style: %08X", _style);
    return SUCCESS;
}



//
// ckIndexParser
//
ckIndexParser::ckIndexParser() 
{   
    _index = -1;    
}

ckError ckIndexParser::parse(ckStream &stream)
{//we don't parse any crap
    char ch;
    string separators = "\n\r\t ";
    string index;
    bool startparent = false;
    bool hasnum = false;
    _str.erase();
   
    while(stream.isMore())
    {
        ch = stream.next();
        if(separators.find(ch) == string::npos)
        {
            if(!startparent)
            {
                 if(ch != '(')
                    _str += ch;
                 else
                     startparent = true;
            }
            else if(startparent && (ch == ')' || !isdigit(ch)))
            {
                break;
            }
            else
            {
                hasnum = true;
                index += ch;
            }
        }
        else
            break;
    }

    if(hasnum)
        _index = atoi(index.c_str());
    else
        _index = -1;

    return (_index != -1 && _str.length() > 0) ? SUCCESS : FAIL;
}