/*
Copyright (C) 2001 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifndef _CK_SCANNER_H_
#define _CK_SCANNER_H_

#include "ckDialog.h"
#include "ckStream.h"

class EscapeSequence
{
public:
    static char getSequence(char ch);
protected:
    static const string _sequence;
    static const string _seqmap;
};

class ckBaseScanner
{
public:
	ckBaseScanner(ckStream &stream, string const &separators);
    virtual ~ckBaseScanner() {}

	ckToken token() const;
	//virtual ckToken peek(string separators = "");
	virtual ckToken accept(string separators = "");
	
    // make sure to return lowercase if case incensitive
	string getString();
	int getInt();
		
    /*
	string getNextString();
	int getNextInt();
    */

    ckStream & getStream() { return _stream;}
protected:
    virtual void determineToken() = 0;

    string _separators;
    string _currentString;
	// we should change it to a reference to stream
	ckStream &_stream;
    ckToken _token;
    

};


class ckScanner : public ckBaseScanner
{
public:
    ckScanner(ckStream &stream, string const &separators);
    virtual ~ckScanner() {}
protected:
    virtual void determineToken();
};

class ckDialogTemplateScanner : public ckScanner
{
public:
    ckDialogTemplateScanner(ckStream &stream);
    virtual ~ckDialogTemplateScanner() {}

protected:
    
};

class ckNodePathScanner : public ckScanner
{
public:
    ckNodePathScanner(ckStream &stream);
    virtual ~ckNodePathScanner() {}
};

class ckStyleScanner : public ckScanner
{
public:
    ckStyleScanner(ckStream &stream);
    virtual ~ckStyleScanner() {}
};

/*
class ckWholeThingScanner : public ckBaseScanner
{
public:
    ckWholeThingScanner(ckStream &stream);
    virtual ~ckWholeThingScanner() {}

	
    // make sure to return lowercase if case incensitive
	virtual ckToken accept(string separators = "");

protected:
    virtual void determineToken() {}

    bool _accepted;
};
*/

#endif
