/*
Copyright (C) 2001 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _CK_STREAM_H_
#define _CK_STREAM_H_

#include "ckDialog.h"

//
// ckStream
//
class ckStream
{
public:
    ckStream() {}
    virtual ~ckStream() {}

    virtual bool isMore() = 0;
    virtual char next() = 0;

    virtual string toString();
};

//
// ckPositionableStream 
//
class ckPositionableStream : public ckStream
{
public:
    ckPositionableStream() {}
    virtual ~ckPositionableStream() {}

    virtual char peek() = 0;
    virtual void setPos(int pos) = 0;
    virtual int getPos() = 0;

    virtual void reset() = 0;
};

//
// ckPositionMemento
//
class ckPositionMemento
{
public:
    ckPositionMemento(ckPositionableStream &stream);

    virtual ~ckPositionMemento();

protected:
    ckPositionableStream &_stream;
    int _pos;
};

class ckStrStream : public ckPositionableStream
{
public:
    ckStrStream(string const &src);

    virtual bool isMore();

    virtual char next();

    virtual char peek();

    virtual void setPos(int pos);

    virtual int getPos();

    virtual void reset();

    // our own 
    virtual string toString();

protected:
    const string _src;
    int _index;
    int _length;
};

/*
//
// ckReverseStrStream
//
class ckReverseStrStream : public ckStrStream
{
public:
    ckReverseStrStream(string const &src)
    :   ckStrStream(src)
    {//alternative would be reverse the string first, then don't need to 
     //implement any virtuals
        _index = _length - 1;
    }

    virtual bool isMore()
    {
        return _index >= 0;
    }

    virtual char next()
    {
        assert(isMore());
        return _src[_index--];
    }

    virtual char peek()
    {
        assert(_index > 0);
        return _src[_index-1];
    }

    virtual void reset()
    {
        _index = _length - 1;
    }

    // our own 
    virtual string toString()
    {
        if(isMore())
        {// might want to actually reverse the whole string
            return _src.substr(0, _index + 1);
        }

        return string();
    }

};
*/

//
// ckStepRCStream
//
class ckStepRCStream : public ckStream
{
public:
    ckStepRCStream(string const &lsconfig);

    virtual ~ckStepRCStream();

    virtual bool isMore();

    virtual char next();
 
protected:
    void _getMore();

    FILE * _file;
    auto_ptr<ckStrStream> _strstream;
    bool _isMore;
    const string _lsconfig;
};

#endif