/*
Copyright (C) 2000 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ckDialog.h"
#include "ckVariables.h"

auto_ptr<ckVariables> ckVariables::_instance;

#include "ckDialog.h"
#include "ckControl.h"
#include "ckStream.h"
#include "ckScanner.h"
#include "ckControlManager.h"


//
// ckVariables
//
ckVariables *ckVariables::instance()
{
    if(_instance.get() == NULL)
    {
        _instance.reset(new ckVariables);
    }

    assert(_instance.get() != NULL);

    return _instance.get();
}

ckVariables::~ckVariables() 
{
    _LSDEBUG1("ckVariables() dtor");
}

ckError ckVariables::translateVariable(string &str, ckControl *control)
{
    if(str.length() > 3 && str[0] == '%' && str[1] == '[' && str[str.length()-1] == ']')
    {
        string temp = str.substr(2, str.length()-3);
        str.erase();

        ckError error = FAIL;

        if(control)
        {
            error = control->retreiveVariable(temp, str);
            /*
            error = control->nodeRecursivePerform(  ckNodePathScanner(ckStrStream(temp)),
                                                    str, ckControl::RETREIVE_VARIABLE);
                                                    */
        }

        if(error != SUCCESS)
        {
            _LSDEBUG1("ckVariables::translateVariable() didn't find it in current node");
            error = ckControlManager::instance()->retreiveVariable(temp, str);
        }

        return error;
    }
    
    return FAIL;
}


//
// ckVariableReplacer
//

string ckVariableReplacer::replace(ckStream &stream)
{
    string expand;
    string var;
    char ch;
    bool bracket = false;

    while(stream.isMore())
    {
        ch = stream.next();
    
        switch(ch)
        {
        case '\\':
            _LSDEBUG1("ckVariableReplacer::replace() got escape sequence");
            if(stream.isMore())
                expand += stream.next();
            break;
        case '%':
            _LSDEBUG1("ckVariableReplacer::replace() got percentage");
            if(stream.isMore() && stream.next() == '[')
            {
                _LSDEBUG1("ckVariableReplacer::replace() starting variable");
                assert(var.empty());
                bracket = true;
            }
            else
                expand += ch;
            break;
        
        case ']':
            _LSDEBUG1("ckVariableReplacer::replace() got back bracket");
            if(bracket)
            {
                _LSDEBUG2("ckVariableReplacer::replace() finish variable: ", var.c_str());
                expand += _replace(var);                    
                var.erase();
                bracket = false;
            }
            else
                expand += ch;
            break; // don't wanna fall through

        default:
            if(bracket)
                var += ch;
            else
                expand += ch;
        }
    }

    if(!var.empty())
    {
        throw ckException(VARIABLE_MISMATCHED, "Variable brackets mismatched");
    }

    return expand;
}


//
// ckGlobalVariableReplacer
//

string ckGlobalVariableReplacer::_replace(string const &var)
{
    string result = var;
    _LSDEBUG2("ckVariableReplacer::replace() about to replace %s", var.c_str());
    ckControlManager::instance()->retreiveVariable(var, result);
    _LSDEBUG2("ckVariableReplacer::replace() replaced as %s", result.c_str());
    return result;
}


