///////////////////////////////////////////////////////
// Lirbrary name: RLC (win32 wrapper classes)
// copyright(c) 2000 Chao-Kuo Lin
/*
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _RL_EXCEPTION_H_
#define _RL_EXCEPTION_H_

#include <windows.h>
#include <commdlg.h>
#include <string>

namespace RL
{
	
/* // use these codes to print out the error codes
::MessageBox(NULL, e.GetMessage(), "Error", MB_OK);
::MessageBox(NULL, e.GetSysMsg(), "Error", MB_OK);
itoa(e.GetErrorCode(), code, 10);
::MessageBox(NULL, code, "Error Code", MB_OK);
*/
///////////////////////////////////////////////////////////
// class Exception
///////////////////////////////////////////////////////////
	class Exception
		{
		public:
			Exception(std::string const &errmsg)
			:	_errMsg(errmsg)
				{}
			
			const char * GetMessage() const { return _errMsg.c_str();}
		protected:
			std::string _errMsg;
		};
///////////////////////////////////////////////////////////
// class CodeException
///////////////////////////////////////////////////////////
// exception that deals with error code and stuff
	class CodeException : public Exception
		{
		public:
			CodeException(std::string const &errMsg, DWORD code)
			:	Exception(errMsg),
				_code(code)
				{}

			DWORD GetErrorCode() const { return _code;}

		protected:
			DWORD _code;
		};
///////////////////////////////////////////////////////////
// class WinException
///////////////////////////////////////////////////////////
	class WinException : public CodeException
		{
			public:
			WinException(std::string const &errMsg)
				:	CodeException(errMsg, ::GetLastError())
				{
					char *psysErrMsg;
					::FormatMessage
					( 
						FORMAT_MESSAGE_ALLOCATE_BUFFER | 
						FORMAT_MESSAGE_FROM_SYSTEM | 
						FORMAT_MESSAGE_IGNORE_INSERTS,
						NULL,
						_code,
						MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
						reinterpret_cast<LPTSTR>(&psysErrMsg),
						0,
						NULL 
					);

					_sysMsg = psysErrMsg;
					::LocalFree(psysErrMsg);

				}				
			~WinException() 
				{}
		
			const char * GetSysMsg() const { return _sysMsg.c_str();}
			
			protected:
			std::string _sysMsg;
		};
///////////////////////////////////
// 	class CommDlgException
///////////////////////////////////
	class CommDlgException : public CodeException
		{
			public:
				CommDlgException(std::string const &errMsg)
				:	CodeException(errMsg, ::CommDlgExtendedError())
					{
					}
		};
				
}	
#endif
